#include <stdio.h>

#include "ResourceManager/ResManager.h"

#include "global.h"
#include "Map.h"
#include "GameObject.h"
#include "GameObjectList.h"


void GameObjectList::register_collisionhandler(EObjectType handles, EObjectType t2){
	if(collisionhandler[handles][t2]){
		printf("collision handler already set for %d|%d\n", handles, t2);
		return;
	}

	collisionhandler[handles][t2] = 1;
	collisionhandler[t2][handles] = 2;
}


GameObjectList::GameObjectList(Map *pmap){
	int i;

	for(i=0; i < MAX_OBJ; i++)
		objlist[i] = NULL;
	objlist_last = 0;

	for(i=0; i < OBJ_LAST; i++){
		for(int j=0; j < OBJ_LAST; j++){
			collisionhandler[i][j] = NULL;
		}
	}

	map = pmap;

	//REGISTER COLLISION HANDLER
	//                        HANDLES               COLLISIONWITH
    register_collisionhandler(OBJ_PLAYER,     OBJ_SPIKES);    //bullet handles bullet-enemy
//	register_collisionhandler(OBJ_PLAYER,      OBJ_LIGHT);
}


GameObjectList::~GameObjectList(){
	clear();
}


void GameObjectList::clear(){
	int i;

	for(i=0; i < objlist_last; i++){
		if(objlist[i]){
			delete objlist[i];
		}
		objlist[i] = NULL;
	}
	objlist_last = 0;
}


bool GameObjectList::add(GameObject *o){
	if(objlist_last < MAX_OBJ){
		objlist[objlist_last]			= o;
		objlist_last++;

		return true;
	}
	else{
		delete o;					//otherwise memory leak!
		return false;
	}
}


void GameObjectList::remove(int i){
	delete objlist[i];

	objlist_last--;
	if(i != objlist_last){	//if not the last object overwrite with last object
	    objlist[i] = objlist[objlist_last];
	}
}


void GameObjectList::update(){
	int i, j, ch;

	for(i=0; i < objlist_last; i++){

		if(objlist[i]->collidesWithMap){
			coldec_obj2map(*objlist[i]); //awful
		}
		else{
			objlist[i]->x += objlist[i]->velx;
			objlist[i]->y += objlist[i]->vely;
		}

		if(objlist[i]->affectedByGravitation){
			objlist[i]->vely += GRAVITY;
		}

		objlist[i]->think();        		//let the current object think. this maybe should be put in a seperate loop after o2o coldec

		if(objlist[i]->removeMe()){
 		    remove(i);                      //and delete the object if neccessary
			i--;							//new object at this position - redo
		}
	}



	//obj2obj colldec (Object Object CollisionDetection)
	for(i=0; i < objlist_last; i++){			//for each object
		for(j=i+1; j < objlist_last; j++){		//test against all other objects (all objects after the current one)
			if(ch = collisionhandler[objlist[i]->type][objlist[j]->type]){	//but only test if one of both objects handles the collision (collisionhandlers are set in GameObjectList()
				if(coldec_obj2obj((*objlist[i]), (*objlist[j]))){
					//printf("collision: %d:%p-%d:%p\n", i,objlist[i], j,objlist[j]);

					//we've got a collision - now check which object handles the collision
					if(ch == 1)
						objlist[i]->collideWithObject(objlist[j]);
					else
						objlist[j]->collideWithObject(objlist[i]);

					printf("col!\n");
				}
			}
		}
	}

}


void GameObjectList::draw(){
	for(int i=0; i < objlist_last; i++){
		objlist[i]->draw();
	}
}



//----------------------------------------------------------------------------------------------------------
#define BETWEEN(t, min, max) (t >= min && t <= max)


bool coldec_obj2obj(GameObject &o1, GameObject &o2){
	int l1;
	int r1;
	int t1;
	int b1;

	int l2;
	int r2;
	int t2;
	int b2;

	if(o1.getVelx() > 0.0f){ //x = left, x+velx = right
		l1 = int(o1.getTopLeftX());			//left
		r1 = int(o1.getTopLeftX()+o1.getW()+o1.getVelx());	//right
	}
	else{
		l1 = int(o1.getTopLeftX()+o1.getVelx());			//left
		r1 = int(o1.getTopLeftX()+o1.getW());	//right
	}
	if(o1.getVely() > 0.0f){
		t1 = int(o1.getTopLeftY());			//top
		b1 = int(o1.getTopLeftY()+o1.getH()+o1.getVely());	//bottom
	}
	else{
		t1 = int(o1.getTopLeftY()+o1.getVely());			//top
		b1 = int(o1.getTopLeftY()+o1.getH());	//bottom
	}


	if(o2.getVelx() > 0.0f){
		l2 = int(o2.getTopLeftX());
		r2 = int(o2.getTopLeftX()+o2.getW()+o2.getVelx());
	}
	else{
		l2 = int(o2.getTopLeftX()+o2.getVelx());
		r2 = int(o2.getTopLeftX()+o2.getW());
	}
	if(o2.getVely() > 0.0f){
		t2 = int(o2.getTopLeftY());
		b2 = int(o2.getTopLeftY()+o2.getH()+o2.getVely());
	}
	else{
		t2 = int(o2.getTopLeftY()+o2.getVely());
		b2 = int(o2.getTopLeftY()+o2.getH());
	}

	//printf("o2: lrtb %d %d %d %d - ", l2, r2, t2, b2);
	//printf("o1: lrtb %d %d %d %d\n", l1, r1, t1, b1);


	//dr bobb's rect-rect collision detection
	//http://www.ddj.com/documents/s=983/ddj9513a/
	if (l1 > r2 || l2 > r1 ||
        t1 > b2 || t2 > b1) {
        return false;
    }
    else {
        return true;
    }

	return false;

/*	int l1 = int(o1.getTopLeftX());			//left
	int r1 = int(o1.getTopLeftX()+o1.getW());	//right
	int t1 = int(o1.getTopLeftY());			//top
	int b1 = int(o1.getTopLeftY()+o1.getH());	//bottom

	int l2 = int(o2.getTopLeftX());
	int r2 = int(o2.getTopLeftX()+o2.getW());
	int t2 = int(o2.getTopLeftY());
	int b2 = int(o2.getTopLeftY()+o2.getH());


	//dr bobb's rect-rect collision detection
	//http://www.ddj.com/documents/s=983/ddj9513a/
	if (l1 > r2 || l2 > r1 ||
        t1 > b2 || t2 > b1) {
        return false;
    }
    else {
        return true;
    }

	return false;
	*/
}