
#pragma once

class GameObjectList{
	public:
		GameObjectList(Map *pmap);
		~GameObjectList();

		void update();				//run game object logic
		void draw();				//draw game objects (->sprites)

		bool add(GameObject *o);	//add a new game object to this list

		void save(FILE *f);
		void load(FILE *f);

		int getLength(){return objlist_last;};	//debug only

	private:
	    Map         *map;
		GameObject	*objlist[MAX_OBJ];						//contains the game objects
		int			objlist_last;							//the first free slot in objlist

		int			collisionhandler[OBJ_LAST][OBJ_LAST];	//which object handles the collision 0... no, 1...o1, 2...o2

		void register_collisionhandler(EObjectType handles, EObjectType t2);
		void remove(int i);
		void clear();
};


//----------------------------------------------------------------------------


/**
 * Pseudo framerate independant method for checking if two objects collide.
 * The collision detection is performed by checking wether the bounding boxes of the
 * two objects intersect.
 * The bounding boxes are extended to cover the whole path the object traveled this frame
 * (taking the velocity into account) thus making the collision detection pseudo
 * framerate independant (and inaccurate if the frames or the velocity on the x and y axis
 * are very large).
 */
bool coldec_obj2obj(GameObject &o1, GameObject &o2);		//object-object coldec