
#include <string>
#include <SDL.h>
#include <SDL_opengl.h>
#include <list>

#include "turska/UI.h"
#include "xml/properties.h"
#include "ResourceManager/ResManager.h"

#include "global.h"
#include "Eventhandler.h"
#include "Gamestate.h"
#include "Map.h"
#include "GameObject.h"
#include "GameObjectList.h"
#include "Light.h"

#include "GamestateStartup.h"

#include "Ghost.h"
#include "Player.h"
#include "Candle.h"
#include "Gold.h"
#include "Spikes.h"

const char *e_candle = "data/images/candle/candle.png";
const char *e_gold = "data/images/gold/gold.png";
const char *e_spikes = "data/images/spikes/spikes.png";

#define img(e) (resmanager->getImage((e)))

void GamestateStartup :: start(){
    //printf("GamestateStartup :: start()\n");

	lightlist			= new LightList();
	map					= new Map("data/maps/map01.map");
	gameobjects			= new GameObjectList(map);

	gameobjects->add(new Ghost(400,400));
	gameobjects->add(new Player(30,200));

	gameobjects->add(new Candle(200,150));
	gameobjects->add(new Candle(400,160));
	gameobjects->add(new Candle(300,500));
	gameobjects->add(new Candle(400,300));
	gameobjects->add(new Candle(700,100));
	gameobjects->add(new Candle(670,240));
	gameobjects->add(new Candle(50,50));
	gameobjects->add(new Candle(50,500));

	gameobjects->add(new Gold(479, 220));
	gameobjects->add(new Gold(178, 220));
	gameobjects->add(new Gold(328, 472));
	gameobjects->add(new Gold(527, 70));

	gameobjects->add(new Spikes(325, 246));
	gameobjects->add(new Spikes(227, 245));
	gameobjects->add(new Spikes(274, 245));
	gameobjects->add(new Spikes(574, 472));
	gameobjects->add(new Spikes(574, 346));
	gameobjects->add(new Spikes(727, 196));
	
	
	editObjects			= false;
	editX				= 0;
	editY				= 0;
	editObject			= img(e_candle);

	this->registerForEvent(SDL_KEYDOWN);
	this->registerForEvent(SDL_MOUSEBUTTONDOWN);
	this->registerForEvent(SDL_MOUSEMOTION);
}    


void GamestateStartup :: run(){
    //printf("GamestateStartup :: run()\n");
	//this->queueGamestateSwitch(new GamestateStartup());

	gameobjects->update();


	graphics->beginRender();

	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);

	glEnable(GL_BLEND);	
			lightlist->draw();

		glBlendFunc(GL_ZERO, GL_SRC_COLOR);		//only draw the map where light is
			map->draw(0,0);

		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	//the eyecandy is drawn as ususal
			gameobjects->draw();

			if(editObjects){
				editObject->draw(UIImage::NORMAL, UIImage::TRANSFORM | UIImage::CENTER, editX, editY);
			}

	glDisable(GL_BLEND);
	
	graphics->endRender();


	SDL_GL_SwapBuffers();

	printf("fps: %.2f\r", getFPS());
}


void GamestateStartup :: stop(){
    //printf("GamestateStartup :: stop()\n");

	delete map;
	delete gameobjects;
}    

void GamestateStartup :: handleEvent(SDL_Event &e){
	if(e.type == SDL_KEYDOWN){
		if(e.key.keysym.sym == SDLK_ESCAPE)
			this->quitGame();
		else if(e.key.keysym.sym == SDLK_e)
			editObjects = !editObjects;
	}
	else if(e.type == SDL_MOUSEMOTION){
		editX = e.motion.x;
		editY = e.motion.y;
	}
	else if(e.type == SDL_MOUSEBUTTONDOWN){
		if(editObjects){
			if(e.button.button == SDL_BUTTON_LEFT){
				if(editObject == img(e_candle))
					gameobjects->add(new Candle(e.button.x, e.button.y));
				else if(editObject == img(e_gold))
					gameobjects->add(new Gold(e.button.x, e.button.y));
				else if(editObject == img(e_spikes))
					gameobjects->add(new Spikes(e.button.x, e.button.y));
				
				printf("e.button.xy %d %d\n", e.button.x, e.button.y);
			}
			else if(e.button.button == SDL_BUTTON_RIGHT){
				if(editObject == img(e_candle))
					editObject = img(e_gold);
				else if(editObject == img(e_gold))
					editObject = img(e_spikes);
				else if(editObject == img(e_spikes))
					editObject = img(e_candle);
			}
		}
	}
}