#include <SDL.h>
#include <list>
#include "ResourceManager/resManager.h"
#include "Eventhandler.h"
#include "Gamestate.h"

#include "global.h"
#include "Map.h"
#include "GameObject.h"
#include "GameObjectList.h"
#include "GamestateStartup.h"

#include "Light.h"
#include "Ghost.h"



Ghost :: Ghost(float nx, float ny): GameObject(nx, ny, resmanager->getProperties("data/objects/ghost.xml"), OBJ_GHOST) ,
									wobbl(0.025f, 5.f, -0.58f, 0.41f){
	this->collidesWithMap			= false;
	this->affectedByGravitation		= false;

	imgGhost = resmanager->getImage("data/images/ghost/ghost.png");
	
	GamestateStartup *gs = (GamestateStartup *)((GamestateManager::getInstance())->getCurrentGamestate());
	gs->getGameobjectList()->add(new GhostCandle(this));

	this->registerForEvent(SDL_MOUSEMOTION);
}

void Ghost :: draw(){
	imgGhost->draw(UIImage::NORMAL, UIImage::TRANSFORM | UIImage::CENTER, x+wobbl.getX(), y+wobbl.getY());
}

void Ghost :: think(){
	wobbl.update();
}



//----------------------------------------------------------------------------------------------------------

void Ghost ::handleEvent(SDL_Event &e){
	if(e.type == SDL_MOUSEMOTION){
		x = e.motion.x;
		y = e.motion.y;
	}
}


Wobbl :: Wobbl(float changespeed, float max, float velx, float vely){
	this->changespeed = changespeed;
	this->max = max;
	this->wobblvelx = velx;
	this->wobblvely = vely;
	this->wobblx = 0;
	this->wobbly = 0;
}

void Wobbl :: update(){
	wobblx += wobblvelx;
	wobbly += wobblvely;

	//printf("wobblvelxy %.2f %.2f\n", wobblvelx, wobblvely);

	if(wobblx > max)
		wobblvelx -= changespeed;
	else if(wobblx < -max)
		wobblvelx += changespeed;
	if(wobbly > max)
		wobblvely -= changespeed;
	else if(wobbly < -max)
		wobblvely += changespeed;
}


//---------------------------------------------------------------------------------------------------
const int distance_to_ghost = 60;
GhostCandle :: GhostCandle(Ghost *ghost) : GameObject(ghost->getX()+distance_to_ghost, ghost->getY(), resmanager->getProperties("data/objects/ghost_candle.xml"), OBJ_GHOSTCANDLE),
											wobbl(0.020f, 4.3f, -0.4f, 0.3f){
	this->ghost = ghost;
	this->imgGhostCandle = resmanager->getImage("data/images/ghost/ghost_candle.png");
	imgLight			 = resmanager->getImage("data/images/ghost/ghost_candle_light.png");
	imgLightWhite		 = resmanager->getImage("data/images/ghost/ghost_candle_light_step2.png");

	lightlist->add(this);

	this->collidesWithMap			= false;
	this->affectedByGravitation		= false;
}


void GhostCandle :: think(){
	this->x = ghost->getX() + distance_to_ghost;
	this->y = ghost->getY();

	wobbl.update();
}


const float lightwobblfactor = 1.8f;
const float lightscalefactor = 0.003f;

void GhostCandle :: draw(){
	imgGhostCandle->draw(UIImage::NORMAL, UIImage::TRANSFORM | UIImage::CENTER, x+ wobbl.getX(), y+ wobbl.getY());
	
	imgLightWhite->setScale(1.0f + wobbl.getX()*lightscalefactor);
	imgLightWhite->draw(UIImage::NORMAL, UIImage::TRANSFORM | UIImage::CENTER | UIImage::SCALE, x+ wobbl.getX()*lightwobblfactor, y+ wobbl.getY()*lightwobblfactor);
}

void GhostCandle :: drawLight(){
	imgLight->setScale(1.0f + wobbl.getX()*lightscalefactor);
	imgLight->draw(UIImage::NORMAL, UIImage::TRANSFORM | UIImage::CENTER | UIImage::SCALE, x+ wobbl.getX()*lightwobblfactor, y+ wobbl.getY()*lightwobblfactor);
}