#include <stdio.h>
#include <map>
#include <string>
#include "turska/UI.h"

#include "ResourceManager/ResManager.h"

#include "Map.h"
#include "global.h"
#include "GameObject.h"
#include "GameObjectList.h"

extern UIImage *imgTile;
extern UIImage *imgTileBlack;

Map *_map = NULL; //for coldec because i'm stupid

Map :: Map(const char* file){
	FILE *mapfile;
	int  i, j, t;

	printf("loading map %s... ", file);

	mapfile = fopen(file, "rb");
	if(mapfile == NULL){
		printf("error opening file - crashing in the most funny ways\n");
		return;
	}

	fread(&width, sizeof(int), 1, mapfile);		//read map width
	fread(&height, sizeof(int), 1, mapfile);	//read map height

	tiles = new TileType[width*height];			//get memory for the map

	xmax = (width-1)*TILESIZE;
	ymax = (height-1)*TILESIZE;

	t = 0;
	for(j = 0; j < width; j++){
		for(i = 0; i < height; i++){
			int c;
			fread(&c, sizeof(int), 1, mapfile);	//read tile
			tiles[t] = (TileType)(c);
			t++;
		}
	}

	tiles2d = new TileType* [height];					//build 2d index array to map data
	for(i=0; i<height; i++){
		tiles2d[i] = &(tiles[i*width]);
	}
	fclose(mapfile);

	
	imgTile				= resmanager->getImage("data/images/tiles/block.png");
	imgTileBlack		= resmanager->getImage("data/images/tiles/black.png");


	_map = this;
	printf("done\n");
}

Map :: ~Map(){
	delete [] tiles;
	delete [] tiles2d;
}

void Map :: draw(int scroll_x, int scroll_y){
	int x, y;	//screen pixel x,y coordinate to draw the current tile to
	int mx, my;
	int t;
	int sx = 0-(scroll_x%TILESIZE);
	int smx = (sx+scroll_x)/TILESIZE;

	for( y = 0-(scroll_y%TILESIZE), my = (y+scroll_y)/TILESIZE;  y < SCREENHEIGHT;  y+=TILESIZE){		//y draw coordinate, the offset is for smooth scrolling
		for( x = sx, mx = smx;  x < SCREENWIDTH;  x+=TILESIZE){	//x draw coordinate
			t = tiles2d[my][mx];		//y/x is changed, mind that!
			//t = tiles[mx+my*width];
			if(t != TT_NOTHING)
				imgTile->draw(x,y);
			else
				imgTileBlack->draw(x,y); //awful, i know
			mx++;
		}
		my++;
	}	
}


void Map :: limitScroll(int &scroll_x, int &scroll_y){

	if(scroll_x < 0)							//too far left?
		scroll_x = 0;
	else if(scroll_x+SCREENWIDTH > width*TILESIZE)		//don't scroll over the map boundaries on the right side (640 is the screenwidth)
		scroll_x = width*TILESIZE-SCREENWIDTH;
	

	if(scroll_y < 0)							//too far up?
		scroll_y = 0;
	
	else if(scroll_y + SCREENHEIGHT > height*TILESIZE)	//bottom map boundaries (480 - screenheight)
		scroll_y = height*TILESIZE-SCREENHEIGHT;
}



//-------------------------------------------------------------------------------------------------------------
//TODO:
// the obj2map code should be changed so that there is no speed limitation.
// actually speeds higher than TILESIZE cause errors, because you might jump over a tile
//
// todo2:
// change the code, so the draw routines don't need the -1 pixel offset (y) (x sometimes)
//
//
// todo3:
// change the code, so we have a lightweight test if vely==1
// or change the code so we can have vely==0 and COL_DOWN occurs on the ground

// todo4:
// add some kind of -sleeping- system, so non moving objects don't get tested

bool c_o2m_cver_r(GameObject &ct);	//obj2map vertical right
bool c_o2m_cver_l(GameObject &ct);	//obj2map vertical left

bool c_o2m_chor_u(GameObject &ct);	//obj2map horizontal up
bool c_o2m_chor_d(GameObject &ct);	//obj2map horizontal down



void coldec_obj2map(GameObject &o){
	ECol		col = COL_NO;
	MapCol      mapcol;

	o.ox = o.x;
	o.oy = o.y;

	o.x = o.x + o.velx;
	o.y = o.y + o.vely;

	mapcol.oldvelx = o.velx;
	mapcol.oldvely = o.vely;
	

	//x axis first (--)
	if(o.velx > 0.0f){		//moving right
		if(o.x > _map->boundx()-o.w){
			o.x = float(_map->boundx()-o.w);
			//o.velx = 0.0f;
			col |= COL_RIGHT;
		}
		else if(c_o2m_cver_r(o))
			col |= COL_RIGHT;
	}		 
	else if(o.velx < 0.0f){	//moving left
		if(o.x < 0.0f){
			o.x = 0.0f;
			//o.velx = 0.0f;
			col |= COL_LEFT;
		}
		else if(c_o2m_cver_l(o))
			col |= COL_LEFT;
	}
	
	//then y axis (|)
	if(o.vely < 0.0f){	//moving up
		if(o.y < 0.0f){
			o.y = 0.0f;
			//o.vely = 0.0f;
			col |= COL_UP;

		}
		else if(c_o2m_chor_u(o)){
			col |= COL_UP;
		}
	}
	else{		//moving down / on ground
		if(o.y > _map->boundy()-o.h){
			o.y = float(_map->boundy()-o.h);
			col |= COL_DOWN;
		}
		else if(c_o2m_chor_d(o)){
			col |= COL_DOWN;
		}
		
	}

	mapcol.side = col;

	if(col & COL_DOWN)
		o.vely = 1.f-GRAVITY;  //test against the ground in the next frame.

	o.collideWithMap(mapcol);	//and let the object react to the collision
}




bool c_o2m_cver_r(GameObject &ct){
	int tileypixels = int( ct.oy-(int(ct.oy)%TILESIZE) );
	int testend = int ( ct.oy + ct.h );

	int tilecoordx = int( (ct.x + ct.w )/TILESIZE );
	int tilecoordy = tileypixels/TILESIZE;

	while(tileypixels <= testend){
		if(_map->map(tilecoordx, tilecoordy) == TT_SOLID){
			ct.x	= float ( tilecoordx*TILESIZE -ct.w-1 );			//move to the edge of the tile (tile on the right -> mind the player width)
			//ct.velx = 0.0f;
			return true;
		}

		tilecoordy++;
		tileypixels += TILESIZE;
	}

	return false;
}

bool c_o2m_cver_l(GameObject &ct){
	int tileypixels = int( ct.oy-(int(ct.oy)%TILESIZE) );
	int testend = int ( ct.oy + ct.h );

	int tilecoordx = int( ct.x/TILESIZE );
	int tilecoordy = tileypixels/TILESIZE;

	while(tileypixels <= testend){
		if(_map->map(tilecoordx, tilecoordy) == TT_SOLID){
			ct.x = float ( (tilecoordx+1)*TILESIZE +1 ) ;			//move to the edge of the tile
			//ct.velx = 0.0f;
			return true;
		}

		tilecoordy++;
		tileypixels += TILESIZE;
	}

	return false;
}



bool c_o2m_chor_u(GameObject &ct){
	int tilexpixels = int ( ct.x-( int ( ct.x ) %TILESIZE ) );
	int testend = int ( ct.x + ct.w );

	int tilecoordy = int ( ct.y/TILESIZE );
	int tilecoordx = tilexpixels/TILESIZE;


	while(tilexpixels <= testend){
		if(_map->map(tilecoordx, tilecoordy) != TT_NOTHING){ //only this changed: when jumping (moving up) we don't want to go through slopes
			ct.y	= float ( (tilecoordy+1)*TILESIZE +1 );
			//ct.vely	= 0.0f;
			return true;	
		}

		tilecoordx++;
		tilexpixels+=TILESIZE;
	}
	return false;
}

bool c_o2m_chor_d(GameObject &ct){
	int tilexpixels = int(ct.x-( int(ct.x)%TILESIZE));	//calculate the x position (pixels!) of the tiles we check against
	int testend = int(ct.x + ct.w);					//calculate the end of testing (just to save the x+w calculation each for loop)

	int tilecoordy = int((ct.y+ct.h)/TILESIZE);			//calculate the y position (map coordinates!) of the tiles we want to test
	int tilecoordx = tilexpixels/TILESIZE;	//calculate map x coordinate for first tile


	//loop while the start point (pixels!) of the test tile is inside the players bounding box
	while(tilexpixels <= testend){
		if(_map->map(tilecoordx, tilecoordy) == TT_SOLID){	//is a solid tile is found at tilecoordx, tilecoordy?
			ct.y	= float (tilecoordy*TILESIZE -ct.h-1);
			//ct.vely = 1.0f;
			return true;
		}

		tilecoordx++;		//increase tile x map coordinate
		tilexpixels+=TILESIZE;	//increase tile x pixel coordinate
	}

	return false;
}

