
#include <stdlib.h>
#include <stdio.h>
#include <SDL.h>
#include <SDL_opengl.h>
#include <list>

#include "turska/UI.h"
#include "xml/properties.h"
#include "ResourceManager/ResManager.h"

#include "enginethingy.h"
#include "Eventhandler.h"
#include "Gamestate.h"

#include "global.h"
#include "Map.h"
#include "GameObject.h"
#include "GameObjectList.h"
#include "GamestateStartup.h"
#include "Light.h"

ResManager *resmanager = NULL;
UIGraphics *graphics = NULL;
LightList *lightlist = NULL;

int main(int argc, char **argv){
	if(!et_init()){
		printf("error while initializing the engine, bye\n");
		return 1;
	}

	graphics = new UIGraphicsOpenGLSDL();
	resmanager = new ResManager(graphics);

	EventhandlerManager *eventhandlermanager = new EventhandlerManager();
	GamestateManager	*gamestatemanager    = new GamestateManager(new GamestateStartup());

	int framestart;
	while(!gamestatemanager->isQuitGameRequested()){
		framestart = SDL_GetTicks();
		SDL_Event event;
		while(SDL_PollEvent(&event)) {
			switch(event.type) {
				case SDL_QUIT:
					gamestatemanager->requestQuit();
				break;
			}
			eventhandlermanager->callEventhandler(event);
		}
		gamestatemanager->runCurrentGamestate();

		while((SDL_GetTicks()-framestart) < 16);
	}

	delete gamestatemanager;
	delete eventhandlermanager;

	delete resmanager;
	delete graphics;

	return 0;
}
